--------------------------------------------------------------------------------
-- Company: 
-- Engineer: Joern Kaipf
--
-- Create Date:    23:39:50 03/27/05
-- Design Name:    
-- Module Name:    control - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity control is
	 generic (
				WIDTH : positive:=8
	 );
    Port ( clk : in std_logic;
           reset : in std_logic;
			  -- uart
			  -- in 
			  TX_complete     : in std_logic;
			  data_present    : in std_logic;
			  uart_data_in	   : in std_logic_vector(7 downto 0);
			  --out
			  send_character  : out std_logic;
			  uart_data_out	: out std_logic_vector(7 downto 0);
			  -- trigger
			  -- in
			  trigger_found   : in std_logic;
			  aqui_finished	: in std_logic;
			  -- out
			  -- xxx

			  -- datareg
			  -- in
			  -- xxx
			  -- out
			  write_regfile   : out std_logic;
			  write_addr	   : out std_logic_vector(2 downto 0);
			  write_data		: out std_logic_vector(WIDTH-1 downto 0);
			  -- ram
			  -- in
			  ram_data_in 		: in std_logic_vector(WIDTH-1 downto 0);
			  read_finished   : in std_logic;
			  -- out
			  reset_ram			: out std_logic;
			  --ram_read_address: out std_logic_vector(10 downto 0);
			  ram_read_en		: out std_logic;
			  ram_data_out		: out std_logic		  
			  );
end control;

architecture Behavioral of control is

   type uart_rx is (IDLE,READ_ADDR,READ_ADDR_1,READ_DATA,WRITE); 
   signal rxstate, rxstate_next : uart_rx; 

	type uart_tx is (IDLE,TX_START,TX_DATA,TX_DATA_1); 
   signal txstate, txstate_next : uart_tx;

	constant RESET_ACTIVE: std_logic:='1';
	signal output_en_addr,output_en_data: std_logic;


begin				 -- uart control
-- rx 

uart_rx_seq:process(clk,reset)
begin
	if (reset=RESET_ACTIVE) then
		rxstate<=IDLE;
	elsif rising_edge(clk) then
		rxstate<=rxstate_next;
	end if;

end process;

uart_rx_com:process(data_present,rxstate)
begin
	-- default statements
	rxstate_next<=rxstate;
	write_regfile<='0';
	output_en_addr<='0';
	output_en_data<='0';

	-- mealy machine
	case rxstate is
		when IDLE =>	
						if (data_present='1') then
							rxstate_next<=READ_ADDR;
						end if;
		when READ_ADDR =>
						output_en_addr<='1';
						rxstate_next<=READ_ADDR_1;
		when READ_ADDR_1 =>
						if (data_present='1') then
							rxstate_next<=READ_DATA;
						end if;
		when READ_DATA =>
						output_en_data<='1';
						rxstate_next<=WRITE;
		when WRITE =>
						write_regfile<='1';
						rxstate_next<=IDLE;
	end case;
end process;

data_output_ff: process(clk,reset)
begin
	if (reset=RESET_ACTIVE) then
		write_data<=(others=>'0');
	elsif rising_edge(clk) then
		if (output_en_data='1') then
			write_data<=uart_data_in;
		end if;	
	end if;
end process;

addr_output_ff: process(clk,reset)
begin
	if (reset=RESET_ACTIVE) then
		write_addr<=(others=>'0');
	elsif rising_edge(clk) then
		if (output_en_addr='1') then
			write_addr<=uart_data_in(2 downto 0);	
		end if;	
	end if;
end process;

-- uart control
-- tx
uart_tx_seq:process(clk,reset)
begin
	if (reset=RESET_ACTIVE) then
		txstate<=IDLE;
	elsif rising_edge(clk) then
		txstate<=txstate_next;
	end if;

end process;

uart_tx_com:process(txstate,aqui_finished,TX_complete,read_finished)
begin
	-- default statements
	txstate_next<=txstate;
	send_character<='0';
	ram_data_out<='0';
	ram_read_en<='0';
	reset_ram<='0';

	-- mealy machine
	case txstate is
		when IDLE =>	reset_ram<='1';
							if (aqui_finished='1') then
								txstate_next<=TX_START;
							end if;
		when TX_START=>
							ram_data_out<='1';	
							send_character<='1';
							txstate_next<=TX_DATA;
		when TX_DATA	=>
							if TX_complete='0' then
								send_character<='1';
								txstate_next<=TX_DATA;
							else
								txstate_next<=TX_DATA_1;
							end if;
		when TX_DATA_1 =>
								txstate_next<=TX_DATA;
								ram_data_out<='1';
								ram_read_en<='1';
								if (read_finished='1') then
									txstate_next<=IDLE;
								end if;
	end case;
end process;

uart_data_out<=ram_data_in;

end Behavioral;
