--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    23:48:24 03/26/05
-- Design Name:    
-- Module Name:    regfile - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity regfile is
	 Generic (
					WIDTH:positive:=8
	 );	
    Port ( clk : in std_logic;
           reset : in std_logic;
           datain : in std_logic_vector(WIDTH-1 downto 0);
           addr : in std_logic_vector(2 downto 0);
           fe_mask : out std_logic_vector(WIDTH-1 downto 0);
           re_mask : out std_logic_vector(WIDTH-1 downto 0);
           datamask : out std_logic_vector(WIDTH-1 downto 0);
           fe_data : out std_logic_vector(WIDTH-1 downto 0);
           re_data : out std_logic_vector(WIDTH-1 downto 0);
           data : out std_logic_vector(WIDTH-1 downto 0);
			  trigger_found : in std_logic;
			  wr : in std_logic;
           arm : out std_logic);
end regfile;

architecture Behavioral of regfile is

signal fe_mask_reg,fe_mask_reg_nxt,
		 re_mask_reg,re_mask_reg_nxt,
		 datamask_reg,datamask_reg_nxt,
		 fe_data_reg,fe_data_reg_nxt,
		 re_data_reg,re_data_reg_nxt,
		 data_reg,data_reg_nxt,
		 control_reg,control_reg_nxt,
		 status_reg,status_reg_nxt: std_logic_vector(WIDTH-1 downto 0);

begin

-- control reg
control_register_seq: process(clk,reset)
begin
	if (reset='1') then
		control_reg<=(others=>'0');
	elsif rising_edge(clk) then
		control_reg<=control_reg_nxt;
	end if;
end process;

control_register_com: process(wr,datain,addr,control_reg,trigger_found)
begin
	control_reg_nxt<=control_reg;
	if (wr='1' and addr="000") then
		control_reg_nxt<=datain;
	end if;
	if (trigger_found='1') then
		control_reg_nxt(7)<='0';
	end if;	
end process;

arm<=control_reg(7);

-- status reg
status_register_seq: process(clk,reset)
begin
	if (reset='1') then
		status_reg<=(others=>'0');
	elsif rising_edge(clk) then
		status_reg<=status_reg_nxt;
	end if;
end process;

status_register_com:process(wr,datain,addr,status_reg)
begin
	status_reg_nxt<=status_reg;
	if (wr='1' and addr="001") then
		status_reg_nxt<=datain;
	end if;
end process;

-- re_mask reg
re_mask_register_seq: process(clk,reset)
begin
	if (reset='1') then
		re_mask_reg<=(others=>'0');
	elsif rising_edge(clk) then
		re_mask_reg<=re_mask_reg_nxt;
	end if;
end process;

re_mask_register_com:process(wr,datain,addr,re_mask_reg)
begin
	re_mask_reg_nxt<=re_mask_reg;
	if (wr='1' and addr="010") then
		re_mask_reg_nxt<=datain;
	end if;
end process;

-- fe_mask reg
fe_mask_register_seq: process(clk,reset)
begin
	if (reset='1') then
		fe_mask_reg<=(others=>'0');
	elsif rising_edge(clk) then
		fe_mask_reg<=fe_mask_reg_nxt;
	end if;
end process;

fe_mask_register_com:process(wr,datain,addr,fe_mask_reg)
begin
	fe_mask_reg_nxt<=fe_mask_reg;
	if (wr='1' and addr="011") then
		fe_mask_reg_nxt<=datain;
	end if;
end process;

-- datamask reg
datamask_register_seq: process(clk,reset)
begin
	if (reset='1') then
		datamask_reg<=(others=>'0');
	elsif rising_edge(clk) then
		datamask_reg<=datamask_reg_nxt;
	end if;
end process;

datamask_register_com:process(wr,datain,addr,datamask_reg)
begin
	datamask_reg_nxt<=datamask_reg;
	if (wr='1' and addr="100") then
		datamask_reg_nxt<=datain;
	end if;
end process;

-- re_data reg
re_data_register_seq: process(clk,reset)
begin
	if (reset='1') then
		re_data_reg<=(others=>'0');
	elsif rising_edge(clk) then
		re_data_reg<=re_data_reg_nxt;
	end if;
end process;

re_data_register_com:process(wr,datain,addr,re_data_reg)
begin
	re_data_reg_nxt<=re_data_reg;
	if (wr='1' and addr="101") then
		re_data_reg_nxt<=datain;
	end if;
end process;

-- fe_data reg
fe_data_register_seq: process(clk,reset)
begin
	if (reset='1') then
		fe_data_reg<=(others=>'0');
	elsif rising_edge(clk) then
		fe_data_reg<=fe_data_reg_nxt;
	end if;
end process;

fe_data_register_com:process(wr,datain,addr,fe_data_reg)
begin
	fe_data_reg_nxt<=fe_data_reg;
	if (wr='1' and addr="110") then
		fe_data_reg_nxt<=datain;
	end if;
end process;

-- data reg
data_register_seq: process(clk,reset)
begin
	if (reset='1') then
		data_reg<=(others=>'0');
	elsif rising_edge(clk) then
		data_reg<=data_reg_nxt;
	end if;
end process;

data_register_com:process(wr,datain,addr,data_reg)
begin
	data_reg_nxt<=data_reg;
	if (wr='1' and addr="111") then
		data_reg_nxt<=datain;
	end if;
end process;

data<=data_reg;
fe_data<=fe_data_reg;
re_data<=re_data_reg;
datamask<=datamask_reg;
re_mask<=re_mask_reg;
fe_mask<=fe_mask_reg;





















end Behavioral;
