library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity trigger is
	generic (
 					WIDTH : positive:=8
	);
	port (
		clk: 		in std_logic;
		datamask:in std_logic_vector(WIDTH-1 downto 0);
		arm     :in std_logic;
		re_mask  :in std_logic_vector(WIDTH-1 downto 0);
		fe_mask  :in std_logic_vector(WIDTH-1 downto 0);
		compdata:in std_logic_vector(WIDTH-1 downto 0);
		comp_re :in std_logic_vector(WIDTH-1 downto 0);
		comp_fe :in std_logic_vector(WIDTH-1 downto 0);
		d: 		in std_logic_vector(WIDTH-1 downto 0);
		data_out:out std_logic_vector(WIDTH-1 downto 0);
		data_en :out std_logic;
		trigout :out std_logic
	);
end trigger;

architecture rtl of trigger is

signal data_trigger,fe_trigger,re_trigger: std_logic;
signal d_sync,d_int,d_int2,d_int3: std_logic_vector(WIDTH-1 downto 0);
signal re_data,fe_data: std_logic_vector(WIDTH-1 downto 0);


begin
	sync: process(clk)
	begin
		if (clk'event and clk='1') then
			d_sync<=d;
			d_int<=d_sync;
			d_int2<=d_int;
			d_int3<=d_int2;
		end if;
	end process;

	process (clk)
	begin
		if (clk'event and clk = '1') then	-- rising edge
			if ((d_int and datamask) = compdata) then
				data_trigger <= '1';
			else
				data_trigger <= '0';
			end if;
		end if;
	end process;
	-- rising edge detection
	process (d_sync,d_int) 
		variable i:integer;
	begin
		for i in 0 to WIDTH-1 loop
			if (d_int(i)='0' and d_sync(i)='1')	then
				re_data(i)<='1';
			else
				re_data(i)<='0';
			end if;
		end loop;
	end process;

	-- falling edge detection
	process (d_sync,d_int) 
		variable i:integer;
	begin
		for i in 0 to WIDTH-1 loop
			if (d_int(i)='1' and d_sync(i)='0')	then
				fe_data(i)<='1';
			else
				fe_data(i)<='0';
			end if;
		end loop;
	end process;

	process (clk)
	begin
		if (clk'event and clk = '1') then	-- rising edge
			if ((re_data and re_mask)=comp_re) then
				re_trigger <= '1';
			else
				re_trigger <= '0';
			end if;
		end if;
	end process;

	process (clk)
	begin
		if (clk'event and clk = '1') then	-- rising edge
			if ((fe_data and fe_mask)=comp_fe) then
				fe_trigger <= '1';
			else
				fe_trigger <= '0';
			end if;
		end if;
	end process;

	trigout<='1' when data_trigger='1' and re_trigger='1' and fe_trigger='1' and arm='1' else '0';

	data_out<=d_int3;

end rtl;
